#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <linux/bpf.h>
#include <sys/socket.h>
#include <sys/syscall.h>
#include <stdbool.h>
#include <errno.h>


int bpf(int cmd, union bpf_attr *attrs)
{
    return syscall(__NR_bpf, cmd, attrs, sizeof(*attrs));
}

// Load the bpf program onto a file descriptor
int load_bpf_prog(void* insn, uint32_t cnt)
{
    char verifier_log_buff[0x200000] = "UNINIT";
    //char verifier_log_buff[0x2000] = {0};
    union bpf_attr prog_attrs =
    {
        .prog_type = BPF_PROG_TYPE_SOCKET_FILTER,
        .insn_cnt = cnt,
        .insns = (uint64_t)insn,
        .license = (uint64_t)"",
        .log_level = 2,
        .log_size = sizeof(verifier_log_buff),
        .log_buf = (uint64_t)verifier_log_buff
    };
    int prog_fd = bpf(BPF_PROG_LOAD, &prog_attrs);

	if (prog_fd < 0) {
		printf("errno is %d\n", errno);

        fprintf(stderr, "%s\n", verifier_log_buff);
		return -1;
	} else {
		return prog_fd;
	}
}

// Try to setup the sockets provided
bool setup_sockets(int prog_fd, int socks[2]) 
{
    if(socketpair(AF_UNIX, SOCK_DGRAM, 0, socks))
    {
		close(socks[0]);
		close(socks[1]);
		return false;
    }

    if(setsockopt(socks[0], SOL_SOCKET, SO_ATTACH_BPF, &prog_fd, sizeof(int)))
    {
		close(socks[0]);
		close(socks[1]);
		return false;
    }

	return true;
}
